 SET all_types              / meat, cereals, wheat, maize, rice, tropicalCereals, oilcrops, soybean, pulses, starchyRoots, pasture /;
 
 SET crop(all_types)                       / wheat, maize, rice, tropicalCereals, oilcrops, soybean, pulses, starchyRoots, pasture /;
 SET crop_less_pasture(crop)               / wheat, maize, rice, tropicalCereals, oilcrops, soybean, pulses, starchyRoots /;
 SET cereal_crop(crop)                     / wheat, maize, rice, tropicalCereals /;
 SET non_cereal_crop(crop)                                                      / oilcrops, soybean, pulses, starchyRoots,              pasture /;
 SET feed_crop(crop)                       / wheat, maize,                        oilcrops, soybean,                       pasture/;
 SET feed_crop_less_pasture(feed_crop)     / wheat, maize,                        oilcrops, soybean          /;
 SET not_feed_crop(crop)                                 / rice, tropicalCereals,                    pulses, starchyRoots /;
 SET import_crop(all_types) / meat,          wheat, maize, rice, tropicalCereals, oilcrops, soybean, pulses, starchyRoots /;
 
 SET location;
 PARAMETER suitableLandArea(location)        areas of land in Mha;
 PARAMETER previousArea(crop, location)      areas for previous timestep in Mha;
 PARAMETER yieldNone(crop, location)         yield in t per ha;
 PARAMETER yieldFertOnly(crop, location)     yield in t per ha;
 PARAMETER yieldIrrigOnly(crop, location)    yield in t per ha;
 PARAMETER yieldBoth(crop, location)         yield in t per ha;
 PARAMETER fertParam(crop, location)         yield response to fertilizer parameter;
 PARAMETER irrigParam(crop, location)        yield response to irrigation parameter;
 PARAMETER demand(all_types)                 in t;
 PARAMETER worldInputEnergy(import_crop)     average input energy from world exports used to determine if we should import or export;
 PARAMETER maxNetImport(import_crop)         maximum net import for each crop based on world market;
 PARAMETER minNetImport(import_crop)         minimum net import for each crop based on world market;
 PARAMETER irrigCost(location)               irrigation cost;
 PARAMETER cropAdj(crop)                     this is the fudge factor that allows the model to fit the observed data
 
 SCALAR meatEfficency                        efficiency of converting feed and pasture into animal products;
 SCALAR landChangeEnergy                     energy required to add ha of agricultural land;
 SCALAR minFeedRate                          minimum rate of feed for producing animal products (why is this needed?);

 SCALAR fitToPreviousAreas                   controlling parameter 0 means dont fit and 1 is to calibrate the cropAdj in the calibration year

*$gdxin "/Users/peteralexander/Documents/R_Workspace/temp/GamsTest/_gams_java_1091455539.gdx"
$gdxin %gdxincname% 
$load location, suitableLandArea, previousArea, demand, landChangeEnergy
$load yieldNone, yieldFertOnly, yieldIrrigOnly, yieldBoth
$load fertParam, irrigParam, worldInputEnergy, maxNetImport, minNetImport
$load meatEfficency, minFeedRate, irrigCost
$load cropAdj, fitToPreviousAreas
$gdxin
    
 SCALAR delta use to smooth power function see 7.5 www.gams.com dd docs solversconopt.pdf / 0.00000000001 /  
     
 SCALAR minDemandPerCereal / 0.1 /
 
 PARAMETER feedDM(crop)  kg DM per kg of feed the conversion from feed to meat is done in the R animal product index
          /   wheat     0.89   
              maize     0.7  
              oilcrops  0.9  
              soybean   0.7  
              pasture   1    / ; 
* pasture yield is done in DM terms
   
 PARAMETER baseCost(crop)  cost per ha before intensity values just made up at the moment
          /   wheat             1.1   
              maize             0.9 
              rice              1.1 
              tropicalCereals   0.9
              oilcrops          0.7 
              soybean           0.6
              pulses            0.5
              starchyRoots      4.0
              pasture           0.1  / ; 
              
 SCALAR fertiliserUnitCost / 1.5 /
 
 PARAMETER previousCropTotalAreas(location);
 previousCropTotalAreas(location) = sum(crop_less_pasture, previousArea(crop_less_pasture, location));
 
 VARIABLES
       area(crop, location)               total area for each crop - Mha
       fertI(crop, location)              fertilizer intensity for each crop - factor between 0 and 1
       irrigI(crop, location)             irrigation intensity for each crop - factor between 0 and 1
       feedAmount(crop)                   amount of feed for each crop - Mt
       netImportAmount(all_types)         net imports of crops and meat - Mt
       yield(crop, location)              yield per area for each crop - t per ha
       unitEnergy(crop, location)         energy per area for each crop - energy
       net_supply(crop)                   supply after exports and feed
       agriLandExpansion(location)        addition agricultural land needed as it must be positive it deliberately does not account for abandonment
       cropIncrease(location)             
       cropDecrease(location)             
       pastureIncrease(location)         
       pastureDecrease(location)          
       energy                             total input energy - energy;
 
 POSITIVE VARIABLE area, fertI, irrigI, feedAmount, agriLandExpansion, cropIncrease, cropDecrease, pastureDecrease, pastureIncrease;
 
 fertI.L(crop, location) = 0.5;
 irrigI.L(crop, location) = 0.5;
 area.L(crop, location) = previousArea(crop, location)
 
 EQUATIONS
       UNIT_ENERGY_EQ(crop, location)                   energy per area
       YIELD_EQ(crop, location)                         yield given chosen intensity
       NON_CEREAL_DEMAND_CONSTRAINT(non_cereal_crop)    satisfy demand for non-cereal crop 
       CEREAL_DEMAND_CONSTRAINT(cereal_crop)            satisfy demand for cereal so that exports can at least be met. Could also allow min. proporation of cereal consumption of each type
       TOTAL_CEREAL_DEMAND_CONSTRAINT                   satisfy demand for cereal
       MEAT_DEMAND_CONSTRAINT                           satisfy demand for meat
       MAX_FERT_INTENSITY_CONSTRAINT(crop, location)    constraint on maximum fertilizer intensity
       MAX_IRRIG_INTENSITY_CONSTRAINT(crop, location)   constraint on maximum irrigation intensity
       TOTAL_LAND_CHANGE_CONSTRAINT(location)           constraint on the rate of land use change 
       NON_FEED_CROP_CONSTRAINT(not_feed_crop)          constraint to set non feed crop feed usage to zero
       MAX_NET_IMPORT_CONSTRAINT(import_crop)           constraint on max net imports
       MIN_NET_IMPORT_CONSTRAINT(import_crop)           constraint on min net imports
       PASTURE_IMPORT_CONSTRAINT                        constraint to not import pasture
       MIN_FEED_CONSTRAINT                              constraint on min feed rate
       NET_SUPPLY_EQ(crop)                              calc net supply for crops
       AGRI_LAND_EXPANSION_CALC(location)               calc agriLandExpansion
       CROP_INCREASE_CALC(location)                     
       CROP_DECREASE_CALC(location)                     
       PASTURE_INCREASE_CONV_CALC(location)             
       PASTURE_DECREASE_CONV_CALC(location)             
       ENERGY_EQ                                        total energy objective function;
 
 UNIT_ENERGY_EQ(crop, location) .. unitEnergy(crop, location) =E=  ( baseCost(crop) * cropAdj(crop) +              
                                                                     fertiliserUnitCost * fertI(crop, location) + 
                                                                     irrigCost(location) * irrigI(crop, location) ) ;
 
 YIELD_EQ(crop, location) .. yield(crop, location) =E= (
               yieldNone(crop, location) + 
               (yieldFertOnly(crop, location) - yieldNone(crop, location)) * ((fertI(crop, location) + delta) ** fertParam(crop, location)) +
               (yieldIrrigOnly(crop, location) - yieldNone(crop, location)) * ((irrigI(crop, location) + delta) ** irrigParam(crop, location)) +
               (yieldBoth(crop, location) + yieldNone(crop, location) - yieldFertOnly(crop, location) - yieldIrrigOnly(crop, location)) *
                                       ((fertI(crop, location) + delta) ** fertParam(crop, location)) * ((irrigI(crop, location) + delta) ** irrigParam(crop, location)) 
            ) * cropAdj(crop);
   
 NET_SUPPLY_EQ(crop) .. net_supply(crop) =E= sum(location, area(crop, location) * yield(crop, location)) - feedAmount(crop) + netImportAmount(crop);
  
 NON_CEREAL_DEMAND_CONSTRAINT(non_cereal_crop) .. net_supply(non_cereal_crop) =G= demand(non_cereal_crop) ;
  
 CEREAL_DEMAND_CONSTRAINT(cereal_crop) .. net_supply(cereal_crop) =G= demand('cereals') * minDemandPerCereal;
  
 TOTAL_CEREAL_DEMAND_CONSTRAINT .. sum(cereal_crop, net_supply(cereal_crop)) =G= demand('cereals');
 
 MEAT_DEMAND_CONSTRAINT .. meatEfficency*sum(feed_crop, feedDM(feed_crop) * feedAmount(feed_crop)) =G= demand('meat') - netImportAmount('meat');
    
 MAX_FERT_INTENSITY_CONSTRAINT(crop, location) .. fertI(crop, location) =L= 1;
 MAX_IRRIG_INTENSITY_CONSTRAINT(crop, location) .. irrigI(crop, location) =L= 1;
 
 TOTAL_LAND_CHANGE_CONSTRAINT(location) .. suitableLandArea(location) =G= sum(crop, area(crop, location));
  
 NON_FEED_CROP_CONSTRAINT(not_feed_crop) .. feedAmount(not_feed_crop) =E= 0;
 
 MAX_NET_IMPORT_CONSTRAINT(import_crop) .. netImportAmount(import_crop) =L= maxNetImport(import_crop);
 MIN_NET_IMPORT_CONSTRAINT(import_crop) .. netImportAmount(import_crop) =G= minNetImport(import_crop);
 PASTURE_IMPORT_CONSTRAINT .. netImportAmount('pasture') =E= 0;
 
 MIN_FEED_CONSTRAINT .. sum(feed_crop_less_pasture, feedDM(feed_crop_less_pasture) * feedAmount(feed_crop_less_pasture)) =G= minFeedRate * (demand('meat') - netImportAmount('meat'));
 
 AGRI_LAND_EXPANSION_CALC(location) .. agriLandExpansion(location) =G= sum(crop, area(crop, location) - previousArea(crop, location)); 
 
 CROP_INCREASE_CALC(location) .. cropIncrease(location) =G= sum(crop_less_pasture, area(crop_less_pasture, location) - previousArea(crop_less_pasture, location));
 CROP_DECREASE_CALC(location) .. cropDecrease(location) =G= -sum(crop_less_pasture, area(crop_less_pasture, location) - previousArea(crop_less_pasture, location));
 PASTURE_INCREASE_CONV_CALC(location) .. pastureIncrease(location) =G= area('pasture', location) - previousArea('pasture', location);
 PASTURE_DECREASE_CONV_CALC(location) .. pastureDecrease(location) =G= -(area('pasture', location) - previousArea('pasture', location));
 
 ENERGY_EQ .. energy =E= (SUM((crop, location), area(crop, location) * unitEnergy(crop, location))         
            + (sum(location, agriLandExpansion(location)) * landChangeEnergy)
            + (sum(location, ((cropIncrease(location) + cropDecrease(location))/previousCropTotalAreas(location)) * landChangeEnergy/10 * previousCropTotalAreas(location)) )
            + (sum(location, ((pastureIncrease(location) + pastureDecrease(location))/previousArea('pasture', location)) * landChangeEnergy/10 *previousArea('pasture', location)) )
            + sum(import_crop, (netImportAmount(import_crop)) * worldInputEnergy(import_crop))) / 1000000;          
 
 MODEL LAND_USE /ALL/ ;
 
 
 scalar basePasture;
 scalar baseCropland;
 scalar count          / 1 /;
 scalar newPasture     / 0 /;
 scalar newCropland    / 0 /;
 
 parameter temp(location);
 
* if fitToPreviousAreas is zero then we just optimise
 if (fitToPreviousAreas eq 0,
     SOLVE LAND_USE USING NLP MINIMIZING energy;
     
     
     
* if fitToPreviousAreas is not zero then we try to find a cropAdj factors that give the previous areas ie we attempt to calibrate
 else
     basePasture = sum(location, previousArea("pasture", location));
     baseCropland = sum((location, crop_less_pasture), previousArea(crop_less_pasture, location));
 
	 while((count le 10 and not (abs(basePasture-newPasture) le 0.1 and abs(baseCropland-newCropland) le 0.1)),
	      SOLVE LAND_USE USING NLP MINIMIZING energy;
	      
	      display pastureIncrease.l, pastureDecrease.l, cropDecrease.l, cropIncrease.l;
	      	      
	      newPasture = sum(location, area.l("pasture", location));
	      newCropland = sum((location, crop_less_pasture), area.l(crop_less_pasture, location));
	      
	      cropAdj(crop_less_pasture) = cropAdj(crop_less_pasture) * newCropland / baseCropland;
	      cropAdj("pasture") = cropAdj("pasture") * newPasture / basePasture;
	      
	      cropAdj(crop) = max(cropAdj(crop), 0.2);
	      cropAdj(crop) = min(cropAdj(crop), 5);
	      
	      display area.l, count, basePasture, baseCropland, newPasture, newCropland, cropAdj;
	      count = count+1 ;
	 );
 );
    
 display agriLandExpansion.l, previousArea, area.l, net_supply.l, demand, feedAmount.l, yield.l, netImportAmount.l;
         
 parameter totalProd(all_types);
 totalProd(crop) = sum(location, area.l(crop, location) * yield.l(crop, location));
 totalProd('meat') = meatEfficency*sum(feed_crop, feedDM(feed_crop) * feedAmount.l(feed_crop));
         
 parameter totalProdCost(all_types);
 parameter totalImportEnergy(all_types);
 parameter feedEnergy(all_types);
 
 totalProdCost(crop) = sum(location, unitEnergy.l(crop, location) * area.l(crop, location));
 totalImportEnergy(import_crop) = netImportAmount.l(import_crop) * worldInputEnergy(import_crop);
 
 feedEnergy(feed_crop)$(netImportAmount.l(feed_crop) gt 0) = (totalProdCost(feed_crop) + totalImportEnergy(feed_crop)) * feedAmount.l(feed_crop) / (totalProd(feed_crop) + netImportAmount.l(feed_crop));
 feedEnergy(feed_crop)$(netImportAmount.l(feed_crop) le 0 and totalProd(feed_crop) gt 0) = totalProdCost(feed_crop) * feedAmount.l(feed_crop) / totalProd(feed_crop);
 
 totalProdCost('meat') = sum(feed_crop, feedEnergy(feed_crop))
 
 parameter totalCropland(location);
 totalCropland(location) = sum(crop_less_pasture, area.l(crop_less_pasture, location));
 
 display totalProdCost, totalProd, totalImportEnergy, feedEnergy, totalProdCost
 
         
 Scalar ms 'model status', ss 'solve status'; 
 ms=LAND_USE.modelstat;
 ss=LAND_USE.solvestat;
