package ac.ed.lurg.country;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.utils.LogWriter;


public class CountryManager {
	private static final int NAME_COL = 0; 
	private static final int CHAR_CODE_COL = 1; 
	private static final int NUM_CODE_COL = 4; 

	private final Map<String, Country> nameMap = new HashMap<String, Country>();
	private final Map<Integer, Country> numCodeMap = new HashMap<Integer, Country>();
	
	private static CountryManager instance = null;

	private static CountryManager getInstance() {
		if(instance == null) {
			instance = new CountryManager();
		}
		return instance;
	}
	
	protected CountryManager() {
		String filename = ModelConfig.COUNTRY_CODES_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, name, charCode;
			int numCode;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 4)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				name = tokens[NAME_COL];
				charCode = tokens[CHAR_CODE_COL];
				numCode = Integer.parseInt(tokens[NUM_CODE_COL]);

				Country c = new Country(name, charCode, numCode);
				nameMap.put(name, c);
				numCodeMap.put(numCode, c);
			} 
			fitReader.close(); 
		
		} catch (Exception e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			e.printStackTrace();
		}
		LogWriter.println("Processed " + filename + ", create " + nameMap.size() + " country codes");

	}
	
	public static Country getForName(String name) {
		Country c = getInstance().nameMap.get(name);
		return c;
	}

	public static Country getForCode(int isoCode) {
		Country c = getInstance().numCodeMap.get(isoCode);
		return c;
	}
}
