package ac.ed.lurg.country.gams;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.Country;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;

public class GamsCountryInput {

	private Country country; // not really required but useful for debugging
	private Map<CommodityType, Double> projectedDemand;
	private Map<CropType, Double> worldInputEnergy;
	private Map<CropType, Double> maxNetImport;
	private Map<CropType, Double> minNetImport;
	private Map<CropType, Double> cropAdjustments;
	private boolean calibrateToObserved;

	// limits to areas for each location and crop
	
/*	private double maxLandUseChange;
	private double meatEfficiency;
	private double minFeedRate;
	private double tradeBarrier;
	private double landChangeEnergy;*/
	
	
	private GamsCountryInput(Country country, Map<CommodityType, Double> projectedDemand, Map<CropType, Double> worldInputEnergy,
			Map<CropType, Double> maxNetImport, Map<CropType, Double> minNetImport, Map<CropType, Double> cropAdjustments, boolean calibrateToObserved) {
		super();
		this.country = country;
		this.projectedDemand = projectedDemand;
		this.worldInputEnergy = worldInputEnergy;
		this.maxNetImport = maxNetImport;
		this.minNetImport = minNetImport;
		this.cropAdjustments = cropAdjustments;
		this.calibrateToObserved = calibrateToObserved;
	}
	
	public static GamsCountryInput createInput(Country country, Map<CommodityType, Double> projectedDemand, Map<CropType, Double> worldInputEnergy,
			Map<CropType, Double> baseNetImport, Map<CropType, Double> maxOfProdOrSupply, Map<CropType, Double> cropAdjustments, boolean calibrateToObserved) {
	
		Map<CropType, Double> maxNetImport;
		Map<CropType, Double> minNetImport;
		
		if (calibrateToObserved) {
			maxNetImport = baseNetImport;
			minNetImport = baseNetImport;

			cropAdjustments = new HashMap<CropType, Double>();  // recreate, but might have been null is this case anyway
			for (CropType c : CropType.getNonMeatTypes()) {
				cropAdjustments.put(c, 1.0);
			}

		}
		else {
			double allowedImportChange = ModelConfig.MAX_IMPORT_CHANGE;
			
			maxNetImport = new HashMap<CropType, Double>();
			minNetImport = new HashMap<CropType, Double>();
			for (Map.Entry<CropType, Double> entry : baseNetImport.entrySet()) {
				CropType c = entry.getKey();
				double change = allowedImportChange * maxOfProdOrSupply.get(c);
				maxNetImport.put(c, entry.getValue() + change);
				minNetImport.put(c, entry.getValue() - change);
			}
		}
		
		return new GamsCountryInput(country, projectedDemand, worldInputEnergy, maxNetImport, minNetImport, cropAdjustments, calibrateToObserved);
	}
	

	public Country getCountry() { 
		return country;
	}
	
	public Map<CommodityType, Double> getProjectedDemand() {
		return projectedDemand;
	}

	public Map<CropType, Double> getWorldInputEnergy() {
		return worldInputEnergy;
	}

	public Map<CropType, Double> getMaxNetImport() {
		return maxNetImport;
	}

	public Map<CropType, Double> getMinNetImport() {
		return minNetImport;
	}

//	public Map<CropType, Double> getMinFeedAmount() {
//		return minFeedAmount;
//	}

	public double getMeatEfficiency() {
		return 1.0;  // this is already handled by the feed conversion efficiency for each animal product
	}

	public double getLandChangeEnergy() {
		return 1.0;
	}

	public double getMinFeedRate() {
		return 0.15;
	}
	
	public Map<CropType, Double> getCropAdjustments() {
		return cropAdjustments;
	}

	public boolean isCalibrateToObserved() {
		return calibrateToObserved;
	}
}
