package ac.ed.lurg.country.gams;

import ac.ed.lurg.country.Country;
import ac.ed.lurg.landuse.AreasItem;
import ac.ed.lurg.landuse.IrrigationCostItem;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.yield.YieldRaster;
import ac.ed.lurg.yield.YieldResponsesItem;
import ac.sac.raster.RasterKey;
import ac.sac.raster.RasterSet;

public class GamsRasterTest extends GamsLocationTest {
	
	public static void main(String[] args)  {
		GamsRasterTest aGamsRasterTest = new GamsRasterTest();
		aGamsRasterTest.run();
	}
	
	private void run() {
		GamsCountryInput countryLevelInputs = GamsCountryInput.createInput(new Country("Test", "TES", 123), getProjectedDemand(), getWorldInputEnergy(), getBaseNetImport(), null, null, true);
		GamsRasterInput input = new GamsRasterInput(getYieldRaster(), getPreviousAreaRaster(), getIrrigationCost(), countryLevelInputs);
		
		GamsRasterOptimiser opti = new GamsRasterOptimiser(input);		
		GamsRasterOutput output = opti.run();
		LogWriter.println(output.toString());
	}

	private static final int X_RANGE = 3;
	private static final int Y_RANGE = 3;

	public YieldRaster getYieldRaster() {
		YieldRaster testYieldRaster= new YieldRaster(null);
		
		for (int i = 0; i<X_RANGE; i++) {
			for (int j = 0; j<Y_RANGE; j++) {
				RasterKey key = new RasterKey(i,j);
				YieldResponsesItem yresp;
				if (i==2 && j==2)
					yresp = getYieldRespItem(0.5*i + 0.3*j, 0.7);
				else
					yresp = getYieldRespItem(2.0/(i+1) + 0.3*j, 1.2);
				testYieldRaster.put(key, yresp);
			}
		}
			
		return testYieldRaster;
	}
	
	public RasterSet<IrrigationCostItem> getIrrigationCost() {
		RasterSet<IrrigationCostItem> testIrrigationCostRaster= new RasterSet<IrrigationCostItem>(null);
		
		for (int i = 0; i<X_RANGE; i++) {
			for (int j = 0; j<Y_RANGE; j++) {
				RasterKey key = new RasterKey(i,j);
				IrrigationCostItem item = new IrrigationCostItem();
				item.setIrrigCost(1.0/(1+i));
				testIrrigationCostRaster.put(key, item);
			}
		}
		
		return testIrrigationCostRaster;
	}

	
	public RasterSet<AreasItem> getPreviousAreaRaster() {
		RasterSet<AreasItem> testAreaRaster= new RasterSet<AreasItem>(null);
		
		for (int i = 0; i<X_RANGE; i++) {
			for (int j = 0; j<Y_RANGE; j++) {
				RasterKey key = new RasterKey(i,j);
				AreasItem areaItem = getAreaItem(i * 20, j * 30);
				testAreaRaster.put(key, areaItem);
			}
		}
		
		return testAreaRaster;
	}
}
