package ac.ed.lurg.demand;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.Country;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;

public class BaseConsumpManager {
	private static final int COUNTRY_COL = 0; 
	private static final int COMMODITY_COL = 3; 
	private static final int BASE_CPC_COL = 7; 
	
	private Map<Country, Map<CommodityType, Double>> baseConsumpMap = new HashMap<Country, Map<CommodityType, Double>>();

	public BaseConsumpManager() {
		super();
		read();
	}
	
	public void read() {
		String filename = ModelConfig.BASELINE_CONSUMP_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, countryName, commodityName;
			double cpc;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 8)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				countryName = tokens[COUNTRY_COL];
				commodityName = tokens[COMMODITY_COL];
				cpc = Double.parseDouble(tokens[BASE_CPC_COL]);

				Country country = CountryManager.getForName(countryName);
				CommodityType commodity = CommodityType.getForFaoName(commodityName);
				
				Map<CommodityType, Double> commodityMap = baseConsumpMap.get(country);
				if (commodityMap == null) {
					commodityMap = new HashMap<CommodityType, Double>();
					baseConsumpMap.put(country, commodityMap);
				}
				
				commodityMap.put(commodity, cpc);
			} 
			fitReader.close(); 
		
		} catch (Exception e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			e.printStackTrace();
		}
		LogWriter.println("Processed " + filename + ", create " + baseConsumpMap.size() + " base consumption values");
	}
	
	public double get(Country country, CommodityType commodity) {
		Map<CommodityType, Double> commodityMap = baseConsumpMap.get(country);

		if (commodityMap == null || !commodityMap.containsKey(commodity)) {
			LogWriter.printlnError("BaseConsumpManager: can't get value for " + commodity + ", " + country);
			return Double.NaN;
		}
		
		Double d = commodityMap.get(commodity);
		return d.doubleValue();
	}
}
