package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.Country;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.ModelFitType;
import ac.ed.lurg.utils.LogWriter;

public class DemandManager {

	private DemandCurveManager demandCurveManager;
	private SspManager sspManager;
	private BaseConsumpManager baseConsumpManager;
	
	private ModelFitType fitType;  // these could be mutable
	private String ssp_scenario;

	public DemandManager (ModelFitType fitType, String ssp_scenario) {
		this.fitType = fitType;
		this.ssp_scenario = ssp_scenario;
		demandCurveManager = new DemandCurveManager();
		sspManager = new SspManager();
		baseConsumpManager = new BaseConsumpManager();
	}
	
	public Map<CommodityType, Double> getDemand(Country country, int year) {
		SspData baseSspData = sspManager.get(ssp_scenario, ModelConfig.BASE_YEAR, country);
		
		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();
		SspData sd = sspManager.get(ssp_scenario, year, country);
		
		LogWriter.println("Got ssp data for " + country.getCountryName() + " of " + sd);
		
		if (sd == null) {
			LogWriter.printlnError("No ssp data for " + ssp_scenario + ", " + year + ", " + country.getCountryName());
			return demandMap;
		}
		
		for (DemandCurve dc : demandCurveManager.get(fitType)) {
			double cpc = dc.getConsumptionPc(baseSspData.getGdpPc(), baseConsumpManager.get(country, dc.getCommodityType()), sd.getGdpPc());
			double d = cpc * sd.getPopulation();
			
			demandMap.put(dc.getCommodityType(), d);
		}
		return demandMap;
	}
	
	public double getPopulation (Country country, int year) {
		SspData sd = sspManager.get(ssp_scenario, year, country);
		if (sd == null) {
			LogWriter.printlnError("No ssp data for " + ssp_scenario + ", " + year + ", " + country.getCountryName());
			return 0;
		}
		else
			return sd.getPopulation();
	}

	public String getModelAndScenarioDescription() {
		return fitType + " " + ssp_scenario;
	}
}
