package ac.ed.lurg.landuse;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import ac.ed.lurg.types.CropToDouble;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.types.LandCoverType;
import ac.sac.raster.RasterItem;

/*
 * Data for one year and country, but it does not know which one
 */
public class AreasItem implements RasterItem {

	private Map<CropType, Double> cropFractions = new HashMap<CropType, Double>();
	private Map<LandCoverType, Double> landCoverAreas = new HashMap<LandCoverType, Double>();
	
	public void setLandCoverAreas(LandCoverItem landCover) {
		if (landCover != null) {
			for (LandCoverType lcType : LandCoverType.values())
				landCoverAreas.put(lcType, landCover.getLandCover(lcType));
		}
	}
	
	public void setLandCoverAreas(AreasItem otherAreasItem) {
		for (Entry<LandCoverType, Double> entry : otherAreasItem.landCoverAreas.entrySet()) {
			landCoverAreas.put(entry.getKey(), entry.getValue());
		}
	}

	/** move areas from one land cover to another, return an residual not possible */
	public double moveAreas(LandCoverType toType, LandCoverType fromType, double changeReq) {
		
		double prevTo = getLandCoverArea(toType);
		double prevFrom = getLandCoverArea(fromType);		
		double actuallyChanged = Math.min(prevFrom, changeReq);
		
		setLandCoverArea(toType, prevTo + actuallyChanged);
		setLandCoverArea(fromType, prevFrom - actuallyChanged);

		return changeReq - actuallyChanged;
	}


	public double getCropArea(CropType c) {
		Double d = getLandCoverArea(LandCoverType.CROPLAND) * getCropFraction(c);
		return d;
	}
	
	public double getCropFraction(CropType c) {
		Double d = cropFractions.get(c);
		return d == null ? 0.0 : d;
	}

	public double getCropFraction(CropType... cropsToFind) {
		double totalFract = 0;
		for (CropType crop : cropsToFind) {
			totalFract += getCropFraction(crop);
		}
		return totalFract;
	}

	
	public void setCropFraction(CropType c, double area) {
		cropFractions.put(c, area);
	}
	
	public double getLandCoverArea(LandCoverType c) {
		Double d = landCoverAreas.get(c);
		return d == null ? 0.0 : d;
	}
	
	public void setLandCoverArea(LandCoverType c, double d) {
		landCoverAreas.put(c, d);
	}
		
	public double getSuitableLand() {
		double d = 0;
		for (LandCoverType l : LandCoverType.values())
			d += getLandCoverArea(l);
		
		return d;
	}
	
	public double getForestToNaturalFraction() {
		double forest = getLandCoverArea(LandCoverType.FOREST);
		double natural = forest + getLandCoverArea(LandCoverType.OTHER_NATURAL);
		double d = forest / natural;
		return d;
	}

	public CropToDouble getCropChanges(AreasItem prevAreaAggItem) {
		CropToDouble changes = new CropToDouble();
		
		for (CropType c : CropType.getCropsLessPasture()) {
			double change = getCropArea(c) - prevAreaAggItem.getCropArea(c);
			changes.put(c, change);
		}
		
		return changes;
	}
}
