package ac.ed.lurg.landuse;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.Country;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;

public class CropUsageData {
	private static final int COUNTRY_COL = 0; 
	private static final int COMMODITY_COL = 2; 
	private static final int FEED_COL = 4; 
	private static final int NET_IMPORT_COL = 5; 
	private static final int PROD_COL = 6; 

	private double feedAmount;
	private double netImports;
	private double prod;
	private double prodCost;
	
	public CropUsageData(double feedAmount, double netImports, double prod, double prodCost) {
		super();
		this.feedAmount = feedAmount;
		this.netImports = netImports;
		this.prod = prod;
		this.prodCost = prodCost;
	}

	public double getFeedAmount() {
		return feedAmount;
	}

	public double getNetImports() {
		return netImports;
	}
	
	public double getProduction() {
		return prod;
	}
	
	public double getProdCost() {
		return prodCost;
	}

	
	public static Map<Country, Map<CropType, CropUsageData>> readCommodityData() {
		Map<Country, Map<CropType, CropUsageData>> commodityMap = new HashMap<Country, Map<CropType, CropUsageData>>();
		String filename = ModelConfig.COMMODITY_DATA_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, countryName, commodityName;
			double feedAmount, netImports, prod;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 6)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				countryName = tokens[COUNTRY_COL];
				commodityName = tokens[COMMODITY_COL];
				feedAmount = Double.parseDouble(tokens[FEED_COL]);
				netImports = Double.parseDouble(tokens[NET_IMPORT_COL]);
				prod = Double.parseDouble(tokens[PROD_COL]);

				Country country = CountryManager.getForName(countryName);
				
				CropUsageData data = new CropUsageData(feedAmount, netImports, prod, Double.NaN);
				Map<CropType, CropUsageData> countryData = commodityMap.get(country);
				if (countryData == null) {
					countryData = new HashMap<CropType, CropUsageData>();
					commodityMap.put(country, countryData);
				}
				
				countryData.put(CropType.getForFaoName(commodityName), data);
			} 
			fitReader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading commodity demand fits");
			e.printStackTrace();
		}
		LogWriter.println("Processed " + filename + ", create " + commodityMap.size() + " country commodity maps values");
		
		return commodityMap;
	}
}
