package ac.ed.lurg.output;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map.Entry;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.landuse.AreasItem;
import ac.ed.lurg.landuse.IntensitiesItem;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.types.LandCoverType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.yield.YieldRaster;
import ac.ed.lurg.yield.YieldResponsesItem;
import ac.sac.raster.RasterKey;
import ac.sac.raster.RasterSet;

public class LpjgOutputer {

	private RasterSet<IntensitiesItem> intensityRaster;
	private RasterSet<AreasItem> cropAreaRaster;
	private YieldRaster yieldSurfaces;
	private int year;

	public LpjgOutputer(int year, RasterSet<IntensitiesItem> intensityRaster, RasterSet<AreasItem> cropAreaRaster, YieldRaster yieldSurfaces) {
		this.year = year;
		this.intensityRaster = intensityRaster;
		this.cropAreaRaster = cropAreaRaster;
		this.yieldSurfaces = yieldSurfaces;
	}

	public void writeOutput() {
		writeLandCoverAndCrop();
	}
	
	public void writeLandCoverAndCrop() {
		BufferedWriter landCoverWriter = null;
		BufferedWriter cropFractWriter = null;

		try {
			String landCoverFileName = ModelConfig.OUTPUT_DIR + File.separator + "LandCoverFract" + year + ".txt";
			landCoverWriter = new BufferedWriter(new FileWriter(landCoverFileName, false));
			landCoverWriter.write("Lon Lat CROPLAND PASTURE NATURAL");
			landCoverWriter.newLine();

			String cropFractionFileName = ModelConfig.OUTPUT_DIR + File.separator + "CropFract" + year + ".txt";
			cropFractWriter = new BufferedWriter(new FileWriter(cropFractionFileName, false));
			cropFractWriter.write("Lon Lat TeWWirr TeSWirr TeCoirr TrRiirr");
			cropFractWriter.newLine();
			
			for (Entry<RasterKey, AreasItem> entry : cropAreaRaster.entrySet()) {
				RasterKey key = entry.getKey();
				AreasItem item = entry.getValue();
				double lat = cropAreaRaster.getXCoordin(key);
				double lon = cropAreaRaster.getYCoordin(key);

				double area = cropAreaRaster.getAreaMha(key);
				double crop = item.getLandCoverArea(LandCoverType.CROPLAND);
				double pasture = item.getLandCoverArea(LandCoverType.PASTURE);
				double forest = item.getLandCoverArea(LandCoverType.FOREST);
				double otherNatural = item.getLandCoverArea(LandCoverType.OTHER_NATURAL);
				landCoverWriter.write(String.format("%.2f %.2f %.4f %.4f %.4f", lat, lon, crop/area, pasture/area, (forest+otherNatural)/area));
				landCoverWriter.newLine();

				boolean isSpringWheat = false;
				YieldResponsesItem yieldItem = yieldSurfaces.get(key);
				if (yieldItem != null)
					isSpringWheat = yieldSurfaces.get(key).isSpringWheat();
				else 
					LogWriter.printlnError("LpjgOutputer: Can't find YieldResponsesItem for " + key);
				
				double lpjWinterWheatFrac = item.getCropFraction(CropType.OILCROPS, isSpringWheat ? null : CropType.WHEAT);
				double lpjSpringWheatFrac = item.getCropFraction(CropType.SOYBEAN, CropType.PULSES, CropType.STARCHY_ROOTS, isSpringWheat ? CropType.WHEAT : null);
				double lpjCornFrac = item.getCropFraction(CropType.MAIZE, CropType.SOYBEAN);
				double lpjRiceFrac = item.getCropFraction(CropType.RICE);
				cropFractWriter.write(String.format("%.2f %.2f %.4f %.4f %.4f %.4f", lat, lon, lpjWinterWheatFrac, lpjSpringWheatFrac, lpjCornFrac, lpjRiceFrac));
				cropFractWriter.newLine();
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		finally {
			if (landCoverWriter != null) {
				try {
					landCoverWriter.close();
				} 
				catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (cropFractWriter != null) {
				try {
					cropFractWriter.close();
				} 
				catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
}
