package ac.ed.lurg.types;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import ac.ed.lurg.utils.LogWriter;

public enum CropType {

	WHEAT("WheatBarleyOats", "wheat"),
	MAIZE("Maize and products", "maize"),
	RICE("Rice (Paddy Equivalent)", "rice"),
	TROPICAL_CEREALS("TropicalCereals", "tropicalCereals"),
	OILCROPS("Oilcrops + (Total)", "oilcrops"),
	SOYBEAN("Soyabeans", "soybean"),
	PULSES("Pulses + (Total)", "pulses"),
	STARCHY_ROOTS("Starchy Roots + (Total)", "starchyRoots"),
	MEAT("meatmilkeggs", "meat", true, true),
	PASTURE("pasture", "pasture", false, false);

	private String faoName;
	private String gamsName;
	private boolean importedCrop;
	private boolean isMeat;

	CropType (String faoName, String gamsName, boolean importedCrop, boolean isMeat) {
		this.faoName = faoName;
		this.gamsName = gamsName;
		this.importedCrop = importedCrop;
		this.isMeat = isMeat;
	}
	
	CropType (String faoName, String gamsName) {
		this(faoName, gamsName, true, false);
	}
	
	public static Collection<CropType> getCropsLessPasture() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (c.importedCrop && !c.isMeat)
				comms.add(c);
		
		return comms;
	}

	public static Collection<CropType> getImportedTypes() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (c.importedCrop)
				comms.add(c);
		
		return comms;
	}
	
	public static Collection<CropType> getNonMeatTypes() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (!c.isMeat)
				comms.add(c);
		
		return comms;
	}

	private static final Map<String, CropType> faoNameCache = new HashMap<String, CropType>();
	private static final Map<String, CropType> gamsNameCache = new HashMap<String, CropType>();
    static {
        for (CropType c : values()) {
        	faoNameCache.put(c.getFaoName(), c);
        	gamsNameCache.put(c.getGamsName(), c);
       }
    }

	public static CropType getForFaoName(String faoName) {
		CropType crop = getFromCache(faoNameCache, faoName);
		if (crop == null) {
			throw new RuntimeException("Can't find CropType for faoName: " + faoName);
		}
		return crop;
	}
	
	public static CropType getForGamsName(String gamsName) {
		return getFromCache(gamsNameCache, gamsName);
	}
	
	private static CropType getFromCache(Map<String, CropType> cache, String name) {
		CropType type = cache.get(name);
		
		if (type == null)
			LogWriter.printlnError("Can't find Item for " + name);
		
		return type;
	}
	
	public static Collection<CropType> getAllItems() {
		return gamsNameCache.values();
	}

	public String getFaoName() {
		return faoName;
	}

	public String getGamsName() {
		return gamsName;
	}
	
	public boolean isImportedCrop() {
		return importedCrop;
	}
}
