package ac.ed.lurg.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;

import ac.ed.lurg.ModelConfig;

public class LogWriter {

	private static LogWriter logWriter;
	private BufferedWriter logFile;
	private boolean suppressStd;

	public static synchronized void println(String s) {
		getLogWriter().print(s, System.out, true);
	}

	public static synchronized void print(String s) {
		getLogWriter().print(s, System.out, false);
	}
	
	public static synchronized void printlnError(String s) {
		getLogWriter().print(s, System.err, true);
	}

	private static LogWriter getLogWriter() {
		if (logWriter == null)
			logWriter = new LogWriter(ModelConfig.OUTPUT_DIR + File.separator + "log.txt");

		return logWriter;
	}

	private LogWriter(String outputFileName) {
		FileWriter fstream;
		try {
			fstream = new FileWriter(outputFileName,false);
			logFile = new BufferedWriter(fstream);
			suppressStd = ModelConfig.SUPPRESS_STD_OUTPUT;
		}
		catch (IOException e) {
			e.printStackTrace();
		}		
	}

	private void print(String s, PrintStream stream, boolean newLine) {
		try {
			logFile.write(s);
			if (newLine) logFile.newLine();
			logFile.flush();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		
		if (!suppressStd) {
			stream.print(s);
			if (newLine) stream.println();
		}
	}

	// probably not called, but never mind as keep stream flushed
	protected void finalize() {
		try {
			logFile.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}
}