package ac.ed.lurg.yield;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.types.YieldType;

public class YieldResponse {
	Map<YieldType, Double> yields = new HashMap<YieldType, Double>();
	private double fertParm;
	private double irrigParm;

	public void setYield(YieldType type, double yield) {
		yields.put(type, yield);
		fertParm = 0;
		irrigParm = 0;
	}
	
	public double getYield(YieldType type) {
		Double d = yields.get(type);
		
		if (d == null) {
		//	LogWriter.printlnError("YieldResponse: No yield found for type " + type);
			return Double.NaN;
		}
		
		return d.doubleValue();
	}		
	
	public double getFertParam() {
		if (fertParm == 0)
			fertParm = calcParam (0, 0.7, 1); // we do have MID fert data, but looks wrong calcParam(getYield(YieldType.NO_FERT_NO_IRRIG), getYield(YieldType.FERT_MID_NO_IRRIG), getYield(YieldType.FERT_MAX_NO_IRRIG));
		
		return fertParm;
	}
	
	public double getIrrigParam() {
		if (irrigParm == 0)
			irrigParm = calcParam (0, 0.8, 1);  // we don't have a mid irrigation figure, so lets assume 60% at mid point
		
		return irrigParm;
	}
	
	private double calcParam(double yMin, double yMid, double yMax) {
		double xMin = 0, xMid = 0.5, xMax = 1;

		if (yMid <= yMin || yMax <= yMid)
			return 1;  // default to linear

		return Math.log((yMid - yMin)/(yMax - yMin)) / Math.log((xMid - xMin)/(xMax - xMin)) / xMax;
	}
}